import os
import re
import shutil

import logging
from datetime import datetime
from os.path import exists

from swagger_coverage.src.models import SwaggerData, EndpointStatisticsHtml
from swagger_coverage.src.utils import sort_requests_results

logger = logging.getLogger("swagger")


class ReportHtml:
    """
    Create html swagger report

    Structure:
    body:
        navbar
        links
        result_table:
            endpoints count result
            progress_bar
            accordions
                diff accordion
    """

    def __init__(
            self, path: str, api_url: str = None, swagger_url: str = None, data=None
    ):
        self.api_url = api_url
        self.swagger_url = swagger_url
        self.data: SwaggerData = data
        self.path: str = path

    _COLOR_RED = "#f59993"
    _COLOR_GREEN = "#d6facf"
    _COLOR_ORANGE = "#f2b85a"

    @staticmethod
    def html(title: str, body: str):
        """
        Create html tags
        """
        return f"""
        <html lang="en"> <head> <meta charset="UTF-8">
            <title>{title}</title>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" # noqa
            rel="stylesheet" integrity="sha384 -1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3" crossorigin="anonymous">
                </head>
                    <body>
                        {body}
                    </body>
                    <script src="./src/script.js"></script>
        </html> """  # noqa

    @staticmethod
    def _navbar():
        date_now = datetime.today().strftime("%Y-%m-%d %H:%M:%S")
        return f"""
            <nav class="navbar navbar-expand-lg navbar-dark bg-dark mb-2">
                <div class="container-fluid">
                    <a class="navbar-brand" href="#">Report created at {date_now}</a>
                        <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarSupportedContent" aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
                            <span class="navbar-toggler-icon"></span>
                        </button>
                    <form class="d-flex">
                        <input id="search" class="form-control me-2" type="search" placeholder="Search" aria-label="Search">
                        <button id="search-button" class="btn btn-outline-success" type="submit">Search</button>
                    </form>
                </div>
            </nav>
        """  # noqa

    def _body(self):
        """
        Create html body
        :return:
        """
        test_statistic, percent_statistic = self.data.summary
        navbar = self._navbar()
        endpoints = self._endpoints_statistics(test_statistic)
        links = self._links()
        progress_bar = self._progress_bar_container(percent_statistic)
        accordion = []
        for endpoint, value in self.data.swagger_data.items():
            res = self._create_accordion(endpoint, value)
            accordion.append(res)
        accordions = "".join(accordion)
        diff_accordion = self._create_diff_accordion_html()
        result_time_request_table = self._result_time_request_table()
        result_request_header = self._request_buttons_colors()
        result_table = self._result_table(
            endpoints_statistics=endpoints,
            progress_bar=progress_bar,
            accordions=accordions,
            diff_accordion=diff_accordion,
            description_request_button=result_request_header,
            request_time_results=result_time_request_table
        )
        html = [navbar, links, result_table]
        return "".join(html)

    @staticmethod
    def _div(class_: str, text: str = ""):
        """
        Create div tag
        """
        return f'<div class="{class_}">{text}</div>'

    @staticmethod
    def _button(id_: str, class_: str, text: str, count: int):
        """
        Create button
        """
        return f'<button type="button" id="{id_}", class="{class_}">{text} <span class="badge bg-secondary">{count}</span></button>'  # noqa

    @staticmethod
    def _button_group(buttons: str):
        return f'<div class="btn-group" role="group" aria-label="Basic mixed styles example">{buttons}</div>'  # noqa

    @staticmethod
    def _link(text: str, link: str) -> str:
        return f'<p class="text-center"">{text}<a href={link}>{link}</a></p>'

    def _links(self):
        api_url = self._link("API url: ", self.api_url)
        swagger_url = self._link("Swagger url: ", self.swagger_url)
        return "".join([api_url, swagger_url])

    def _endpoints_statistics(self, data: EndpointStatisticsHtml):
        """
        Create file description, like url and other.
        """
        button_endpoints = self._button(
            id_="all", class_="btn btn-primary", text="All", count=data.endpoints
        )
        button_checked_endpoints = self._button(
            id_="success",
            class_="btn btn-success",
            text="Checked",
            count=data.checked_endpoints,
        )
        button_not_checked_endpoints = self._button(
            id_="not-checked",
            class_="btn btn-danger",
            text="Not checked",
            count=data.not_checked_endpoints,
        )
        button_not_added_endpoints = self._button(
            id_="not-added",
            class_="btn btn-warning",
            text="Not added",
            count=data.not_added_endpoints,
        )
        buttons = "".join(
            [
                button_endpoints,
                button_checked_endpoints,
                button_not_checked_endpoints,
                button_not_added_endpoints,
            ]
        )
        data = self._button_group(buttons)
        content = self._div(class_="d-flex justify-content-center", text=data)
        return content

    def _request_buttons_colors(self):
        """
        Create file description, like url and other.
        """
        button_checked_endpoints = self._button(
            id_="success",
            class_="btn btn-light",
            text="< 0.5 sec",
            count='',
        )
        button_not_checked_endpoints = self._button(
            id_="not-checked",
            class_="btn btn-warning",
            text="0.5-1 sec",
            count='',
        )
        button_not_added_endpoints = self._button(
            id_="not-added",
            class_="btn btn-danger",
            text="> 1 sec",
            count='',
        )
        buttons = "".join(
            [
                button_checked_endpoints,
                button_not_checked_endpoints,
                button_not_added_endpoints,
            ]
        )
        data = self._button_group(buttons)
        content = self._div(class_="d-flex justify-content-center", text=data)
        return content

    def _progress_bar_container(self, data):
        success = self._progress_bar(result_class="success", percent=data.success)
        failed = self._progress_bar(result_class="danger", percent=data.failed)
        result = "".join([success, failed])
        return f"""
            <div class="container px-4 py-1">
                <div class="row gx-5">
                    <div class="progress gx-0">
                    {result}
                    </div>
                </div>
            </div>"""
        pass

    @staticmethod
    def _progress_bar(result_class: str, percent: float) -> str:
        return f"""
            <div class="progress-bar bg-{result_class} px-5"
                role="progressbar" style="width: {percent}%" aria-valuenow=20 aria-valuemin="0" aria-valuemax="100">{percent}%
            </div>
            """  # noqa

    @staticmethod
    def _result_table(
            progress_bar: str,
            accordions: str,
            endpoints_statistics: str,
            diff_accordion: str,
            description_request_button: str,
            request_time_results: str
    ):
        return f"""
        <div class="container px-5">
            <div class="row gx-0">
            <p class="text-center fs-2" >
            Swagger API report
            </p>
            {endpoints_statistics}
            {progress_bar}
            <div id=accordions>
                {accordions}
                {diff_accordion}
            </div>
            </div>
            <p class="text-center fs-2" >
            Request average time 
            </p>
            {description_request_button}
            {request_time_results}
        </div>
        """

    def _result_time_request_table(self):
        return f"""
          <table class="table">
  <thead>
    <tr>
      <th scope="col">#</th>
      <th scope="col">Name</th>
      <th scope="col">Result(seconds)</th>
    </tr>
  </thead>
  <tbody>
    {self._create_request_data_table()}
  </tbody>
</table>
           """

    def _create_request_data_table(self) -> str:
        results = sort_requests_results(self.data.swagger_data)
        table_body = []
        for count, res in enumerate(results):
            table_body.append('<thead style="font-style: normal">\n')
            table_body.append(f'<tr class="table-{res.get("color")}" style="font-style: normal">\n')
            table_body.append(f'<th scope="col">{count + 1}</th>\n')
            table_body.append(f'<th scope="col">{res.get("name")}</th>\n')
            table_body.append(f'<th scope="col">{res.get("results")}</th>\n')
            table_body.append('</tr>\n')
            table_body.append('</thead>\n')
        return ''.join(table_body)

    @staticmethod
    def _create_section(status: dict) -> str:
        st, result = list(status.items())[0]
        color = "green" if result else "red"
        return f"""
                <section>
                    <p style="color:{color};">{st}</p>
                </section>
            """

    def _create_accordion(self, endpoint, value, color: str = None):
        """
        id, color, description, sections
        """
        text_description = self._camel_terms_to_str(endpoint)
        description = (
            f"<b>{text_description}</b> " f'({value.get("method")} {value.get("path")})'
        )
        is_checked_list = [list(status.values())[0] for status in value.get("statuses")]
        if color is None:
            color = self._COLOR_RED if False in is_checked_list else self._COLOR_GREEN

        table_rows = []
        for count, row in enumerate(value.get("statuses")):
            res = self._create_table_body(count + 1, row)
            table_rows.append(res)

        desc = value.get("description", "-")
        if desc is None:
            desc = "-"

        avg_execution = value.get("time_executions")
        if avg_execution is None:
            summery_time_exc = "-"
        else:
            summery_time_exc = sum(avg_execution) / len(avg_execution)
        return f"""
        <div class="accordion-item">
            <h2 class="accordion-header" id="{endpoint}">
                    <button class="accordion-button collapsed" type="button"
                        data-bs-toggle="collapse" data-bs-target="#flush-collapseOne"
                        aria-expanded="false" aria-controls="flush-collapseOne"
                        style="background-color: {color};">
                        {description}
                    </button>
            </h2>
        <div class="accordion-collapse collapse" aria-labelledby="flush-headingOne"
            data-bs-parent="#accordionFlushExample" data-state="collapse">
                <div class="accordion-body">
                    <section>
                        <b>Description:</b> {desc}<br>
                        <b>Average execution:</b> {str(summery_time_exc)[:5]} seconds <br>
                        {self._create_table(''.join(table_rows))}
                    </section>
                </div>
            </div>
        </div>
                """  # noqa

    def _create_table(self, rows):
        """
        Create table in accordion
        """
        return f""" <table class="table">
                      <thead>
                        <tr>
                          <th scope="col">#</th>
                          <th scope="col">Status code</th>
                          <th scope="col">Number of calls</th>
                        </tr>
                      </thead>
                      {rows}
                    </table>
        """

    def _create_table_body(self, count: int, status: dict) -> str:
        st, result = list(status.items())[0]
        color = "green" if result else "red"
        return f"""<tbody>
                        <tr>
                          <th scope="row">{count}</th>
                          <td><p style="color:{color};">{st}</p></td>
                          <td>{result}</td>
                        </tr>
                      </tbody>
                      """

    @staticmethod
    def _create_accordion_diff(endpoint, value, color: str = None):
        """
        id, color, description, sections
        """
        description = "Missing endpoints"

        return f"""
        <div class="accordion-item">
            <h2 class="accordion-header" id="{endpoint}">
                    <button class="accordion-button collapsed" type="button"
                        data-bs-toggle="collapse" data-bs-target="#flush-collapseOne"
                        aria-expanded="false" aria-controls="flush-collapseOne"
                        style="background-color: {color};">
                        {description}
                    </button>
            </h2>
        <div class="accordion-collapse collapse" aria-labelledby="flush-headingOne"
            data-bs-parent="#accordionFlushExample" data-state="collapse">
                <div class="accordion-body"> <section>
                    {value} </section>
        </div>
            </div>
                </div>
                """  # noqa

    @staticmethod
    def _area(text: str):
        return f"""
                <textarea class="form-control" rows="3">{text}</textarea>
                """

    def _create_diff_accordion_html(self):
        """
        create diff html
        """
        diff_text = self._create_diff_text(self.data.diff)
        return self._create_accordion_diff(
            endpoint="endpoint", color=self._COLOR_ORANGE, value=self._area(diff_text)
        )

    @staticmethod
    def _create_diff_text(diff: dict) -> str:
        """
        Create diff text for data_swagger.yaml
        """
        text = []
        spaces = "  "
        for key, values in diff.items():
            desc = values.get("description", "-")
            if desc is None:
                desc = "-"
            text.append(f"{key}:\n")
            text.append(f"{spaces}description: {desc}\n")
            text.append(f'{spaces}method: {values.get("method")}\n')
            text.append(f'{spaces}path: {values.get("path")}\n')
            text.append(f"{spaces}statuses:\n")
            text.append(f"{spaces}- 200\n")
            text.append(f"{spaces}- 400\n")
            text.append(f'{spaces}tag: {values.get("tag")} \n')
        return "".join(text)

    @staticmethod
    def _create_dir(dir_path):
        if not exists(dir_path):
            os.mkdir(dir_path)

    def _copy_src_folder(self):
        """
        Copy folder with css and js files
        :return:
        """
        # create path from library
        src_dir = os.path.dirname(os.path.abspath(__file__))
        src_library_path = os.path.join(src_dir, "files", "script.js")
        # create dir in project
        src_path_js = os.path.join(self.path, "src", "script.js")
        self._create_dir(os.path.join(self.path, "src"))
        shutil.copyfile(src_library_path, src_path_js)

    @staticmethod
    def _camel_terms_to_str(value: str):
        """
        From CamelCase to String
        """
        result = re.findall(
            "[A-Z][a-z]+|[0-9A-Z]+(?=[A-Z][a-z])|[0-9A-Z]{2,}|[a-z0-9]{2,}|[a-zA-Z0-9]",
            value,
        )  # noqa
        result[0] = result[0].capitalize()
        return " ".join(result)

    def save_html(self, file_name: str = "index.html", is_copy=True):
        """
        Save html with swagger check diff
        """
        body = self._body()
        html = self.html("Swagger coverage API", body)
        with open(os.path.join(self.path, file_name), "w") as outfile:
            outfile.write(html)
        if is_copy:
            self._copy_src_folder()
        logger.info(
            f"The swagger report was successfully saved to the folder: " f"{self.path}"
        )



class EndpointStatisticsHtml:
    def __init__(
        self,
        endpoints: int,
        checked_endpoints: int,
        not_checked_endpoints: int,
        not_added_endpoints: int,
    ):
        self.endpoints = endpoints
        self.checked_endpoints = checked_endpoints
        self.not_checked_endpoints = not_checked_endpoints
        self.not_added_endpoints = not_added_endpoints


class PercentStatistic:
    def __init__(self, success: str, failed: str):
        self.success = success
        self.failed = failed


class DescriptionHtml:
    def __init__(self, api_url: str, swagger_url: str):
        self.api_url = api_url
        self.swagger_url = swagger_url


class SwaggerData:
    def __init__(
        self, swagger_data: dict = None, diff: dict = None, summary: dict = None
    ):
        self.swagger_data = swagger_data
        self.diff = diff
        self.summary = summary



from typing import List


def sort_requests_results(data: dict) -> List:
    """
    Sort request result
    """
    results = []
    for key, value in data.items():
        if value.get('time_executions') is not None:
            name = f"{value.get('tag')} ({value.get('method')} {value.get('path')})"
            avg_time = sum(value.get('time_executions'))/len(value.get('time_executions'))
            if avg_time > 1:
                color = 'danger'
            elif avg_time > 0.5:
                color = 'warning'
            else:
                color = 'light'
            results.append({'name': name, 'results': avg_time, 'color': color})
    sorted_results = sorted(results, key=lambda d: d['results'], reverse=True)
    return sorted_results



import time
from functools import wraps

from swagger_coverage.src.check_data import SwaggerChecker


def swagger(key):
    def wrapper(function):
        @wraps(function)
        def inner(*args, **kwargs):
            start = time.time()
            res = function(*args, **kwargs)
            end = time.time()
            time_execution = end - start
            try:
                SwaggerChecker().swagger_check(key, res, time_execution)
                return res
            except AttributeError:
                return res

        return inner

    return wrapper



import copy

from requests import Response

from swagger_coverage.src.coverage import SwaggerCoverage


class SwaggerChecker:
    def __init__(self):
        self.data = SwaggerCoverage().data

    def swagger_check(self, key: str, res: Response, time_execution: float) -> None:
        """
        Try to check response status code and swagger data
        """
        dict_data = copy.deepcopy(self.data)
        SwaggerCoverage().data = self._set_check_result(
            key, res.status_code, dict_data, time_execution
        )

    @staticmethod
    def _set_check_result(
        key: str, status_code: int, data, time_execution: float
    ) -> dict:
        """
        Set check result
        """
        endpoint = data.swagger_data.get(key)
        if endpoint:
            if endpoint.get("time_executions") is None:
                endpoint["time_executions"] = [time_execution]
            else:
                t_exc = endpoint.get("time_executions")
                t_exc.append(time_execution)
            statuses = endpoint.get("statuses")
            for status in statuses:
                for key in status.keys():
                    if key == status_code:
                        status[key] = status.get(key, 0) + 1

                        return data
        return data



import json
import os
from os.path import exists

import yaml


class FileOperation:
    @staticmethod
    def create_dir(dir_name: str):
        if not exists(dir_name):
            os.mkdir(dir_name)

    @staticmethod
    def save_yaml(data: dict, path_file: str):
        """
        save yaml
        """
        with open(path_file, "w") as outfile:
            yaml.Dumper.ignore_aliases = lambda self, data: True
            yaml.dump(data, outfile, default_flow_style=False, Dumper=yaml.Dumper)

    @staticmethod
    def save_json(data: dict, path_file: str):
        """
        save json file
        """
        json_data = json.dumps(data)
        file_to_write = path_file + '/results.json'
        with open(file_to_write, 'w') as f:
            json.dump(data, f)


    @staticmethod
    def load_yaml(path_to_file: str) -> dict:
        with open(path_to_file, "r") as stream:
            return yaml.safe_load(stream)

    @staticmethod
    def get_path_to_file(path: str, file_name: str) -> str:
        return os.path.join(path, file_name)



import copy

from swagger_coverage.src.models import (
    PercentStatistic,
    EndpointStatisticsHtml,
    SwaggerData,
)


class DataDiff:
    def result_diff(self, data: SwaggerData, paths, prepare_data):
        """
        Get swagger check result, need for build html report
        """
        data.diff = self._swagger_diff(paths, prepare_data)
        data.summary = self._get_summary(data.diff, data)
        return data

    @staticmethod
    def _swagger_diff(paths, prepare_data):
        """
        Get swagger diff
        """
        diff = {k: prepare_data[k] for k in set(prepare_data) - set(paths)}
        return diff

    def _get_summary(
        self, diff: dict, data
    ) -> [EndpointStatisticsHtml, PercentStatistic]:
        """
        Calculate report summary
        """
        count_success = 0
        count_of_unverified = 0
        data = copy.deepcopy(data.swagger_data)
        for key, value in data.items():
            is_checked_list = [
                list(status.values())[0] for status in value.get("statuses")
            ]
            count_success += len([status for status in is_checked_list if status > 0])
            count_of_unverified += len(
                [status for status in is_checked_list if status == 0]
            )
        count_diff = len(list(diff.items()))
        count_total = count_success + count_of_unverified
        # get percent
        percentage_success = self._percentage(count_success, count_total)
        percentage_unverified = self._percentage(count_of_unverified, count_total)
        return (
            EndpointStatisticsHtml(
                count_total, count_success, count_of_unverified, count_diff
            ),
            PercentStatistic(percentage_success, percentage_unverified),
        )

    @staticmethod
    def _percentage(part, whole) -> str:
        """
        Calculate percentage of verified statuses
        """
        res = 100 * float(part) / float(whole)
        return format(res, ".1f")



import logging
import sys

l_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"


def setup(level: str):
    """
    Run logger
    :param level: logger level 'CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'
    """
    set_level = getattr(logging, level)
    formatter = logging.Formatter(l_format)
    logger = logging.getLogger()
    # Logger INFO
    info_handler = logging.StreamHandler(sys.stdout)
    info_handler.setLevel(set_level)
    info_handler.setFormatter(formatter)
    info_handler.addFilter(lambda rec: rec.levelno == set_level)
    logger.addHandler(info_handler)
    # Logger ERROR
    err_handler = logging.StreamHandler(sys.stderr)
    err_handler.setLevel(logging.ERROR)
    err_handler.setFormatter(formatter)
    err_handler.addFilter(lambda rec: rec.levelno == logging.ERROR)
    logger.addHandler(err_handler)



import dataclasses
import json
import logging
import os
from os.path import exists

from swagger_coverage.src.models import SwaggerData
from swagger_coverage.src.report import ReportHtml
from swagger_coverage.src.singltone_like import Singleton
from swagger_coverage.src.swagger_diff import DataDiff
from swagger_coverage.src.files import FileOperation
from swagger_coverage.src.prepare_data import PrepareData

logger = logging.getLogger("swagger")


class SwaggerCoverage(metaclass=Singleton):
    """
    Tool for calculating status coverage in api tests
    First, you need to create a file for verification, for example

        swagger = SwaggerCoverage(url='https://test.com', status_codes=[200, 400],
            path='/report')
        swagger.create_coverage_data()


    Where you need to specify a link to the swagger.

    Second, use html report

        swagger_rep.create_report()

    Report will be created in the folder './report/index.html'
    """

    _TEST_SWAGGER_FILE_NAME = "data_swagger.yaml"
    _SWAGGER_REPORT_DIR = "swagger_report"
    _DEFAULT_STATUS_CODE = [200, 400, 401, 403]

    def __init__(
        self,
        url: str = None,
        api_url: str = None,
        status_codes: list = None,
        path: str = None,
    ):
        """
        Init Swagger Coverage
        :param url:
        :param api_url:
        :param status_codes:
        :param path:
        """
        self.url = url
        self.api_url = api_url
        self.paths = None
        self.prepare_data = None
        if status_codes is None:
            self.status_codes = self._DEFAULT_STATUS_CODE
        else:
            self.status_codes = [int(s) for s in status_codes]
        self.path = self._prepare_path(path=path)
        self.data = SwaggerData()
        self.report = ReportHtml
        self.file = FileOperation
        self.prepare = PrepareData
        self.diff = DataDiff()

    def _prepare_path(self, path: str) -> str:
        """
        param path: parth to report dir, for example '/report'
        :return: return path to report dir
        """
        if path is None:
            parent_dir = os.path.abspath(os.path.abspath(os.curdir))
            return os.path.join(parent_dir, self._SWAGGER_REPORT_DIR)
        parent_dir = os.path.abspath(os.path.abspath(os.curdir))
        path = path[1:] if path.find("/") == 0 else path
        return os.path.join(parent_dir, path)

    def create_coverage_data(self, file_name: str = _TEST_SWAGGER_FILE_NAME):
        """
        Create coverage data
        """
        self.file.create_dir(self.path)
        path_to_file = self.file.get_path_to_file(self.path, file_name)
        if exists(path_to_file) is False:
            self.paths = self.prepare.load_swagger(self.url)
            self.prepare_data = self.prepare.prepare_swagger_data(
                data=self.paths, status_codes=self.status_codes
            )
            self.file.save_yaml(path_file=path_to_file, data=self.prepare_data)
        dict_data = self.file.load_yaml(path_to_file=path_to_file)
        data_new = self.prepare.prepare_check_file_data(dict_data)
        self.data.swagger_data = data_new
        logger.info("Data coverage creation completed successfully")

    def save_results(self):
        """
        Save result in json file
        :return:
        """
        paths = self.prepare.load_swagger(self.url)
        if self.prepare_data is None:
            self.prepare_data = self.prepare.prepare_swagger_data(
                data=paths, status_codes=self.status_codes
            )
        self.report(
            api_url=self.api_url,
            swagger_url=self.url,
            data=self.diff.result_diff(
                self.data, self.data.swagger_data, self.prepare_data
            ),
            path=self.path,
        )
        self.file.save_json(self.data.swagger_data, self.path)
        pass

    def create_report(self):
        """
        Save result in html file
        :return:
        """
        paths = self.prepare.load_swagger(self.url)
        if self.prepare_data is None:
            self.prepare_data = self.prepare.prepare_swagger_data(
                data=paths, status_codes=self.status_codes
            )
        reporter = self.report(
            api_url=self.api_url,
            swagger_url=self.url,
            data=self.diff.result_diff(
                self.data, self.data.swagger_data, self.prepare_data
            ),
            path=self.path,
        )
        reporter.save_html()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['swagger_coverage', 'swagger_coverage.scripts', 'swagger_coverage.src']

package_data = \
{'': ['*'], 'swagger_coverage.src': ['files/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'PyYAML>=6.0,<7.0',
 'attrs>=21.4.0,<22.0.0',
 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['swagger_coverage = '
                     'swagger_coverage.scripts.swagger_report:main']}

setup_kwargs = {
    'name': 'test-swagger-coverage',
    'version': '0.1.43',
    'description': 'Swagger coverage for API tests',
    'long_description': '# swagger-coverage\n[![Tests](https://github.com/berpress/swagger-coverage/actions/workflows/python-app.yml/badge.svg)](https://github.com/berpress/swagger-coverage/actions/workflows/python-app.yml)\n![versions](https://img.shields.io/pypi/pyversions/pybadges.svg)\n[![Maintainability](https://api.codeclimate.com/v1/badges/45afb8b947b1c7e9cec8/maintainability)](https://codeclimate.com/github/berpress/swagger-coverage/maintainability)\n[![Test Coverage](https://api.codeclimate.com/v1/badges/45afb8b947b1c7e9cec8/test_coverage)](https://codeclimate.com/github/berpress/swagger-coverage/test_coverage)\n[![PyPI version](https://badge.fury.io/py/test-swagger-coverage.svg)](https://badge.fury.io/py/test-swagger-coverage)\n[![Downloads](https://pepy.tech/badge/test-swagger-coverage)](https://pepy.tech/project/test-swagger-coverage)\n\nAbout\n------------\n\nSwagger coverage report helps the QA automation and developer to get a simple API coverage report for endpoints tests\n\n![](https://github.com/berpress/python-api-tests/blob/main/images/swagger_report_2.png)\n\nInstallation\n------------\n\nYou can install ``test-swagger-coverage`` via `pip`_ from `PyPI`_::\n\n    $ pip install test-swagger-coverage\n\nor with poetry\n\n    $ poetry add test-swagger-coverage\n\nHow it works\n------------\nWe take a swagger as data for testing coverage and, based on it, we create a file that will be the settings for our tests. The file can be created automatically or manually.\n\nNext, we set up api calls in our tests (we wrap them with decorators, see examples) and at the end of testing we generate html report.\nWe will check which endpoints were called and what statuses we checked.\n\nWe can\'t always trust our swagger, so you can manually set the status of the codes yourself, which need to be checked.\n\nExamples\n------------\n\nFirst, we need a link to your swagger. For example,  let\'s take this  https://app.swaggerhub.com/apis-docs/berpress/flask-rest-api/1.0.0 (see more about this api https://github.com/berpress/flask-restful-api),\nand take url to yaml/yml/json swagger file - https://api.swaggerhub.com/apis/berpress/flask-rest-api/1.0.0\n\nNext, in our project, we need to create a file describing our endpoints, which our tests will use to generate a coverage report.\n\nWe can do it automatically via the command line\n\n    $ swagger_coverage https://api.swaggerhub.com/apis/berpress/flask-rest-api/1.0.0\n\nResult\n\n    $ 2022-04-15 11:22:37 INFO Start load swagger https://api.swaggerhub.com/apis/berpress/flask-rest-api/1.0.0\n    $ 2022-04-15 11:22:38 INFO The swagger report was successfully saved to the folder: /Users/user/Documents/git/python-api-tests/swagger_report\n\n\n\nThe **swagger_report** directory will be created\nand a **data_swagger.yaml** file will appear inside, which will be the settings for building a test coverage report\n\nThe **data_swagger.yaml** file looks something like this\n\n\n ```\n ...\nregUser:\n  description: null\n  method: POST\n  path: /register\n  statuses:\n  - 201  <---- change from 200 to 201\n  - 400\n  - 401\n  - 403\n  tag: register\n  ...\n ```\n\nwhere **regUser** is the unique id of our endpoint\n\n**statuses** is a list of statuses that we will check (that they were called).\nYou can customize this list yourself.\n\nOnly we will check 201 status, as described in the user registration swagger. So I will add it.\n\n\nLet\'s create a simple test and build a report. For requests, you will use the **requests** library.\n\n```python\nimport requests\nfrom swagger_coverage.src.coverage import SwaggerCoverage\nfrom swagger_coverage.src.deco import swagger\n\n# swagger data preparation\nswagger_url = "https://api.swaggerhub.com/apis/berpress/flask-rest-api/1.0.0"\napi_url = "https://api.swaggerhub.com/apis/"\npath=\'/report\' # path to swagger coverage report\nswagger_rep = SwaggerCoverage(api_url=api_url, url=swagger_url, path=path)\nswagger_rep.create_coverage_data()\n\n\n# function to call a request to the server\n@swagger("regUser")\ndef register_user(payload: dict):\n    return requests.post(\'https://stores-tests-api.herokuapp.com/register\',\n                         json=payload)\n\n\n# test\ndata = {"username": "test2023@test.com", "password": "Password"}\nresponse = register_user(data)\nassert response.status_code == 201\n\n# create report\nswagger_rep.create_report()\n\n```\n**swagger data preparation**: Prepare our file data_swagger.yaml, it will be created automatically.\n\n**function to call a request to the server**:  We will write a user registration call. Declaring a function with a decorator **@swagger("regUser")**.\n**"regUser"** taken from file **data_swagger.yaml**, this is unique id of our endpoint.\n\n**test**: run the test\n\n**create report**: create a report.\n\n\nAfter that, in the folder **swagger_report** we will receive a report **index.html**.\n\nLet\'s see it\n\n![](https://github.com/berpress/python-api-tests/blob/main/images/swagger_register.png)\n\nAs you can see, we have increased the counter of verified endpoints\n\nIf you use **pytest**, add this code in conftest.py\n\n```python\n@pytest.fixture(scope="session", autouse=True)\ndef swagger_checker(request):\n    url = request.config.getoption("--swagger-url")\n    url_api = request.config.getoption("--api-url")\n    path = \'/report\'\n    swagger = SwaggerCoverage(api_url=url_api, url=url, path=path)\n    swagger.create_coverage_data()\n    yield\n    swagger.create_report()\n```\n\nMore example with pytest and API tests https://github.com/berpress/python-api-tests\n\nReport example [https://github.com/berpress/python-api-tests/tree/main/swagger_report](https://github.com/berpress/python-api-tests/tree/main/report)\n',
    'author': 'alexanderlozovoy',
    'author_email': 'berpress@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/berpress/swagger-coverage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Dict

import requests

import logging

import yaml

logger = logging.getLogger("swagger")


class PrepareData:
    @staticmethod
    def load_swagger(url: str) -> Dict:
        """
        Load and get swagger data
        """
        logger.info(f"Start load swagger {url}")
        res = requests.get(url)
        if res.status_code == 200:
            try:
                data = yaml.safe_load(res.text)
                return data.get("paths")
            except Exception:
                raise ValueError("Couldn't load yaml")
        else:
            logger.error(f"Couldn't get the file, status code is {res.status_code}")

    @staticmethod
    def prepare_swagger_data(data, status_codes: list) -> dict:
        """
        Preparing data for tests
        :param status_codes:
        :param data:
        :return: swagger dict
        """
        res_dict = {}
        for key, value in data.items():
            list_values = list(value.values())
            for values in list_values:
                res_dict[values.get("operationId")] = []
            for k, v in value.items():
                res_dict[v.get("operationId")] = {
                    "method": k.upper(),
                    "description": v.get("description"),
                    "path": key,
                    "statuses": status_codes,
                    "tag": v.get("tags")[0],
                    "time": []
                }
        return res_dict

    @staticmethod
    def prepare_check_file_data(data: dict) -> dict:
        """
        Prepare data for check
        """
        for k, value in data.items():
            statuses = value.get("statuses")
            if statuses:
                new_statuses = []
                for s in statuses:
                    new_statuses.append({s: 0})
                value["statuses"] = new_statuses
        return data



#!/usr/bin/env python
import argparse
import logging.config
import sys

from swagger_coverage.src.coverage import SwaggerCoverage
from swagger_coverage.src.logger import setup

logger = logging.getLogger("swagger")

LOGGER_LEVEL = ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"]


def main():
    parser = argparse.ArgumentParser(description="Swagger coverage")
    parser.add_argument("url", type=str)
    parser.add_argument(
        "-s",
        "--status-codes",
        nargs="+",
        help="set status codes to " "create a report",
        type=str,
    )
    parser.add_argument(
        "-l", "--level", help=f"set logging level {LOGGER_LEVEL}", type=str
    )
    parser.add_argument("-p", "--path", help="path to report folder", type=str)
    args = parser.parse_args()
    url = args.url
    path = args.path
    status_codes = args.status_codes
    logging_level = args.level
    if logging_level is None:
        logging_level = "INFO"
    elif logging_level not in LOGGER_LEVEL:
        raise ValueError("Check logger level")
    setup(logging_level)
    logger.setLevel(logging_level)
    swagger = SwaggerCoverage(url=url, status_codes=status_codes, path=path)
    swagger.create_coverage_data()


if __name__ == "__main__":
    try:
        main()
    except ValueError:
        sys.exit(2)
